from asgiref.sync import async_to_sync
from django.contrib.auth.models import User
from django.test import Client, TestCase

from bookshelf import services
from bookshelf.models import Book, Category


class FindMyCategoriesServiceTests(TestCase):
    def test_find_my_categories_should_return_my_categories(
        self,
    ):
        me = User.objects.create_user(
            "me", "me@example.com", "password"
        )
        another = User.objects.create_user(
            "another",
            "another@example.com",
            "password",
        )
        my_category = Category.objects.create(
            name="日本文学", user=me
        )
        Category.objects.create(
            name="アメリカ文学", user=another
        )
        sync_find_my_categories = async_to_sync(
            services.find_my_categories
        )
        self.assertEqual(
            sync_find_my_categories(me.id),
            [my_category],
        )


class FindMyBooksOfCategoriesServiceTests(TestCase):
    """指定されたカテゴリのユーザーが登録した本を取得するサービスのテスト"""

    def test_find_my_books_of_categories_should_return_categorized_books(
        self,
    ):
        me = User.objects.create_user(
            "me", "me@example.com", "password"
        )
        j_literature = Category.objects.create(
            name="日本文学", user=me
        )
        a_literature = Category.objects.create(
            name="アメリカ文学", user=me
        )

        j_literature_book1 = Book.objects.create(
            name="こころ", category=j_literature
        )
        j_literature_book2 = Book.objects.create(
            name="舞姫", category=j_literature
        )
        Book.objects.create(
            name="白鯨", category=a_literature
        )
        sync_find_my_books_of_categories = (
            async_to_sync(
                services.find_my_books_of_categories
            )
        )
        self.assertCountEqual(
            sync_find_my_books_of_categories(
                j_literature.id
            ),
            [
                j_literature_book1,
                j_literature_book2,
            ],
        )


class BookshelvesGraphQLTests(TestCase):
    def setUp(self):
        self.client = Client()
        self.garphql_url = "/graphql/"

    def test_graphql_should_return_result(self):
        me = User.objects.create_user(
            "me", "me@example.com", "password"
        )
        j_literature = Category.objects.create(
            name="日本文学", user=me
        )
        a_literature = Category.objects.create(
            name="アメリカ文学", user=me
        )
        Book.objects.create(
            name="草枕", category=j_literature
        )
        Book.objects.create(
            name="舞姫", category=j_literature
        )
        Book.objects.create(
            name="白鯨", category=a_literature
        )
        Book.objects.create(
            name="緋文字", category=a_literature
        )
        self.client.login(
            username="me", password="password"
        )
        query = """
        query TestQuery {
            me {
                id
                username
                categories {
                    id
                    name
                    books {
                        id
                        name
                    }
                }
            }
        }
        """
        response = self.client.post(
            self.garphql_url,
            {"query": query},
            content_type="application/json",
        )
        self.assertEqual(
            response.json()["data"],
            {
                "me": {
                    "id": "1",
                    "username": "me",
                    "categories": [
                        {
                            "id": "1",
                            "name": "日本文学",
                            "books": [
                                {
                                    "id": "1",
                                    "name": "草枕",
                                },
                                {
                                    "id": "2",
                                    "name": "舞姫",
                                },
                            ],
                        },
                        {
                            "id": "2",
                            "name": "アメリカ文学",
                            "books": [
                                {
                                    "id": "3",
                                    "name": "白鯨",
                                },
                                {
                                    "id": "4",
                                    "name": "緋文字",
                                },
                            ],
                        },
                    ],
                }
            },
        )
